import Flutter
import UIKit

public class SwiftWmcPersonalizedPlugin: NSObject, FlutterPlugin {
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "wmc_personalized_plugin", binaryMessenger: registrar.messenger())
        let instance = SwiftWmcPersonalizedPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if call.method == "initialize"{
            
            guard let args = call.arguments else {
                return
            }
            
            if let myArgs = args as? [String: Any],
               let apikey = myArgs["apiKey"] as? String{
                
                
                let now = timestamp()
                
                ApiWmcModule.shared.initialize(apiKey: apikey) { (response) in
                    switch(response){
                    case .Success(let r):
                        let finished = self.timestamp() - now
                        
                        print("ApiWMC/Initialize: timeExecute initialize: \(finished) miliseconds")
                        
                        result(r)
                    case .Error(_):
                        let finished = self.timestamp() - now
                        
                        print("ApiWMC/Initialize: Error initialize: \(finished) miliseconds")
                        result(FlutterError(code: "-1", message: "Error initialize WMC", details: nil))
                    }
                }
                
            }else {
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                                        "flutter arguments in method: (sendParams)", details: nil))
            }
            
        }else if call.method == "session"{
            let now = timestamp()
            
            ApiWmcModule.shared.getUser() { (response) in
                
                switch(response){
                case .Success(let r):
                    let finished = self.timestamp() - now
                    
                    print("ApiWMC/session: timeExecute session: \(finished) miliseconds")
                    result(r)
                case .Error(_):
                    let finished = self.timestamp() - now
                    
                    print("ApiWMC/session: Error session: \(finished) miliseconds")
                    result(FlutterError(code: "-1", message: "Error request WMC", details: nil))
                }
                
            }
        }else if call.method == "request"{
            guard let args = call.arguments else {
                return
            }
            
            if let myArgs = args as? [String: Any],
               let uri = myArgs["uri"] as? String,
               let sessionJson = myArgs["session"] as? String,
               let method = myArgs["method"] as? String,
               let options = myArgs["options"] as? Dictionary<String,String>,
               let data = myArgs["data"] as? Dictionary<String,String>{
                
                let datajson = Data(sessionJson.utf8)
                let session = try? JSONDecoder().decode(Session.self, from: datajson)
                
                let now = timestamp()
                
                
                ApiWmcModule.shared.request(session:session!,uri: uri, method: method, options: options, data: data) { (response) in
                    
                    switch(response){
                    case .Success(let r):
                        let finished = self.timestamp() - now
                        
                        print("ApiWMC/request: timeExecute request: \(finished) miliseconds")
                        result(r)
                    case .Error(_):
                        let finished = self.timestamp() - now
                        
                        print("ApiWMC/request: Error request: \(finished) miliseconds")
                        result(FlutterError(code: "-1", message: "Error request WMC", details: nil))
                    }
                    
                }
            }else{
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                                        "flutter arguments in method: (sendParams)", details: nil))
            }
            
        }else if call.method == "post"{
            guard let args = call.arguments else {
                return
            }
            
            if let myArgs = args as? [String: Any],
               let uri = myArgs["uri"] as? String,
               let sessionJson = myArgs["session"] as? String,
               let options = myArgs["options"] as? Dictionary<String,String>,
               let data = myArgs["data"] as? String{
                
                let now = timestamp()
                
                
                let datajson = Data(sessionJson.utf8)
                let session = try? JSONDecoder().decode(Session.self, from: datajson)
                ApiWmcModule.shared.post(session:session!,uri: uri, options: options, data: data) { (response) in
                    
                    switch(response){
                    case .Success(let r):
                        let finished = self.timestamp() - now
                        
                        print("ApiWMC/post: timeExecute post: \(finished) miliseconds")
                        result(r)
                    case .Error(_):
                        let finished = self.timestamp() - now
                        
                        print("ApiWMC/post: Error post: \(finished) miliseconds")
                        result(FlutterError(code: "-1", message: "Error request WMC", details: nil))
                    }
                    
                }
            }else{
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                                        "flutter arguments in method: (sendParams)", details: nil))
            }
            
        }else if call.method == "put"{
            guard let args = call.arguments else {
                return
            }
            
            if let myArgs = args as? [String: Any],
               let uri = myArgs["uri"] as? String,
               let sessionJson = myArgs["session"] as? String,
               let options = myArgs["options"] as? Dictionary<String,String>,
               let data = myArgs["data"] as? String{
                
                let now = timestamp()
                
                
                let datajson = Data(sessionJson.utf8)
                let session = try? JSONDecoder().decode(Session.self, from: datajson)
                
                ApiWmcModule.shared.put(session:session!,uri: uri, options: options, data: data) { (response) in
                    
                    switch(response){
                    case .Success(let r):
                        let finished = self.timestamp() - now
                        
                        print("ApiWMC/put: timeExecute put: \(finished) miliseconds")
                        result(r)
                    case .Error(_):
                        let finished = self.timestamp() - now
                        
                        print("ApiWMC/put: Error put: \(finished) miliseconds")
                        result(FlutterError(code: "-1", message: "Error request WMC", details: nil))
                    }
                    
                }
            }else{
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                                        "flutter arguments in method: (sendParams)", details: nil))
            }
            
        }else if call.method == "resource"{
            guard let args = call.arguments else {
                return
            }
            
            if let myArgs = args as? [String: Any],
               let uri = myArgs["uri"] as? String,
               let sessionJson = myArgs["session"] as? String{
                
                let now = timestamp()
                
                
                let datajson = Data(sessionJson.utf8)
                let session = try? JSONDecoder().decode(Session.self, from: datajson)
                
                ApiWmcModule.shared.getResource(session:session!,uri: uri) { (response) in
                    
                    switch(response){
                    case .Success(let r):
                        let finished = self.timestamp() - now
                        
                        print("ApiWMC/getResource: timeExecute resource: \(finished) miliseconds")
                        result(r)
                    case .Error(_):
                        let finished = self.timestamp() - now
                        
                        print("ApiWMC/getResource: Error resource: \(finished) miliseconds")
                        result(FlutterError(code: "-1", message: "Error request WMC", details: nil))
                    }
                    
                }
            }else{
                result(FlutterError(code: "-1", message: "iOS could not extract " +
                                        "flutter arguments in method: (sendParams)", details: nil))
            }
        }else if call.method == "connectionData"{
            
            ApiWmcModule.shared.connectionData() { (response) in 
                switch(response){
                case .Success(let r):
                    result(r)
                case .Error(_):
                    result(FlutterError(code: "-1", message: "Error request WMC", details: nil))
                }
                
            }
        }else{
            result(FlutterMethodNotImplemented)
        } 
    }
    
    
    func timestamp() -> Double{
        return Double(Date().timeIntervalSince1970)
    }
}
