import 'dart:async';
import 'dart:typed_data';
import 'package:flutter/services.dart';

class WmcPersonalizedPlugin {
  static const MethodChannel _channel =
      const MethodChannel('wmc_personalized_plugin'); 

  /*
   * Inicializacion de WMC
   */
  static Future<String> initialize(String apikey) async { 
    String _result;
    try {
      final String result =
            await _channel.invokeMethod('initialize',{"apiKey":apikey});
             
      _result = result;
    } on PlatformException catch (e) {
      _result = "${e.message}";
    }  
    return _result;
  }

  /*
   * Método que obtiene sessiones
   */
  static Future<String> session() async {
    String _session;
    try {
      final String result =
      await _channel.invokeMethod('session');

      _session = result;
    } on PlatformException catch (e) {
      _session = e.message;
    }
    return _session;
  }

  /*
   * Método que realiza una solicitud request (GET,DELETE)
   */
  static Future<Map> request(String session,String uri,String method,Map options,Map data) async {
      Map _result;
      try {
        final Map result =
              await _channel.invokeMethod('request',{
              "uri":uri,
              "session":session,
              "method":method,
              "options":options,
              "data": data
          });
              
        _result = result;
      } on PlatformException catch (e) {
        _result = {'error': e.message};
      }  
      return _result;
    }
  /*
   * Método que realiza una solicitud POST
   */

  static Future<Map> post(String session,String uri,Map options,String json) async {
      Map _result;
      try {
        final Map result =
              await _channel.invokeMethod('post',{
              "uri":uri,
              "session":session,
              "options":options,
              "data": json
          });
              
        _result = result;
      } on PlatformException catch (e) {
        _result = {'error': e.message};
      }  
      return _result;
    }


  /*
   * Método que realiza una solicitud PUT
   */

  static Future<Map> put(String session,String uri,Map options,String json) async {
      Map _result;
      try {
        final Map result =
              await _channel.invokeMethod('put',{
              "uri":uri,
              "session":session,
              "options":options,
              "data": json
          });
              
        _result = result;
      } on PlatformException catch (e) {
        _result = {'error': e.message};
      }  
      return _result;
    }

    /*
   * Método que obtiene información de conexión
   */

  static Future<Map> getConnectionData() async { 
      Map _result;
      try {
        final Map result =
              await _channel.invokeMethod('connectionData');
              
        _result = result;
      } on PlatformException catch (e) {
        _result = {'error': e.message};
      }  
      return _result;
    }

    

    
   /*
   * Método que obtiene la información en bytes de un recursos (Imagen)
   */
  static Future<Uint8List> resource(String session,String uri) async {
    Uint8List _result;
    try {
      final Uint8List result =
            await _channel.invokeMethod('resource',{"session":session,"uri":uri});
             
      _result = result;
    } on PlatformException catch (e) {
      _result = null;
    }
    return _result;
  }
}
